package com.huimv.env.common.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.huimv.env.common.entity.BaseFarm;
import com.huimv.env.common.entity.EnergyElectricity;
import com.huimv.env.common.entity.EnergyEnvDevice;
import com.huimv.env.common.entity.EnergyWater;
import com.huimv.env.common.entity.vo.*;
import com.huimv.env.common.mapper.*;
import com.huimv.env.common.service.IEnergyEnvDeviceService;
import com.huimv.env.common.utils.ExcelImportSheet;
import com.huimv.env.common.utils.Print;
import com.huimv.env.common.utils.Result;
import com.huimv.env.common.utils.ResultCode;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-02-21
 */
@Service
public class EnergyEnvDeviceServiceImpl extends ServiceImpl<EnergyEnvDeviceMapper, EnergyEnvDevice> implements IEnergyEnvDeviceService {
    @Autowired
    private EnergyEnvDeviceMapper energyEnvDeviceMapper;
    @Autowired
    private BaseFarmMapper farmMapper;

    @Autowired
    private IEnergyEnvDeviceService envDeviceService;
    @Autowired
    private EnergyWaterMapper waterMapper;
    @Autowired
    private EnergyElectricityMapper electricityMapper;

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String type = paramsMap.get("type");
        String status = paramsMap.get("status");
        String deviceCode = paramsMap.get("deviceCode");
        String farmId = paramsMap.get("farmId");
        String pageSize = paramsMap.get("pageSize");
        String pageNo = paramsMap.get("pageNum");
        if (pageSize == null || pageSize == "") {
            pageSize = "10";
        }
        if (pageNo == null || pageNo == "") {
            pageNo = "1";
        }
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(StringUtils.isNotBlank(deviceCode), "device_code", deviceCode);
        queryWrapper.eq(StringUtils.isNotBlank(status), "device_status", status);
        queryWrapper.eq("farm_id", farmId);
        if ("1".equals(type)) {
            queryWrapper.in("device_type", 1, 2);
        }
        if ("3".equals(type)) {
            queryWrapper.eq("device_type", 3);
        }
        Page<EnergyEnvDevice> page = new Page(Integer.parseInt(pageNo), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, energyEnvDeviceMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(HttpServletRequest httpServletRequest, EnergyEnvDevice energyEnvDevice) {
        QueryWrapper<BaseFarm> farmQueryWrapper = new QueryWrapper<>();
        farmQueryWrapper.eq("id", energyEnvDevice.getFarmId());
        BaseFarm farm = farmMapper.selectOne(farmQueryWrapper);
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.clear();
        queryWrapper.eq("device_code", energyEnvDevice.getDeviceCode()).eq("farm_id", energyEnvDevice.getFarmId());
        Integer count1 = energyEnvDeviceMapper.selectCount(queryWrapper);
        if (count1 > 0) {
            return new Result(10001, "设备id已存在", false);
        }
        queryWrapper.clear();
        queryWrapper.eq("unit_id", energyEnvDevice.getUnitId()).eq("farm_id", energyEnvDevice.getFarmId());
        Integer count2 = energyEnvDeviceMapper.selectCount(queryWrapper);
        if (count2 > 0) {
            return new Result(10001, "该单元已绑定设备", false);
        }
        energyEnvDevice.setRemark(farm.getFarmName());
        energyEnvDevice.setUnitName(energyEnvDevice.getDeviceName());
        energyEnvDeviceMapper.insert(energyEnvDevice);
        return new Result(ResultCode.SUCCESS, "添加成功");
    }

    @Override
    public Result edit(HttpServletRequest httpServletRequest, EnergyEnvDevice energyEnvDevice) {
        QueryWrapper<EnergyEnvDevice> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.clear();
        queryWrapper1.eq("device_code", energyEnvDevice.getDeviceCode()).eq("farm_id", energyEnvDevice.getFarmId()).ne("id", energyEnvDevice.getId());
        Integer count1 = energyEnvDeviceMapper.selectCount(queryWrapper1);
        if (count1 > 0) {
            return new Result(10001, "设备id已存在", false);
        }
        queryWrapper1.clear();
        queryWrapper1.eq("unit_id", energyEnvDevice.getUnitId()).eq("farm_id", energyEnvDevice.getFarmId()).ne("id", energyEnvDevice.getId());
        Integer count2 = energyEnvDeviceMapper.selectCount(queryWrapper1);
        if (count2 > 0) {
            return new Result(10001, "该单元已绑定设备", false);
        }
        energyEnvDeviceMapper.updateById(energyEnvDevice);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result delete(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String id = paramsMap.get("id");
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", id).eq("farm_id", farmId);
        energyEnvDeviceMapper.delete(queryWrapper);
        return new Result(ResultCode.SUCCESS, "删除成功");
    }

    @Override
    public Result bandingUnitId(HttpServletRequest httpServletRequest, EnergyEnvDevice envDevice) {
        Integer unitId = envDevice.getUnitId();
        int count = this.count(new QueryWrapper<EnergyEnvDevice>().eq("unit_id", unitId));
        if (count > 0) {
            return new Result(10001, "绑定失败，该栋舍已有设备", false);
        }
        return new Result(10000, "绑定成功", true);
    }

    @Override
    public Result deviceCount(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        Integer count = energyEnvDeviceMapper.selectCount(queryWrapper);
        JSONObject jsonObject = new JSONObject();
        if (count == 0) {
            jsonObject.put("DeviceCount", 0);
            jsonObject.put("OnDeviceCount", 0);
            jsonObject.put("OffDeviceCount", 0);
            jsonObject.put("OnDeviceRate", 0);
        } else {
            QueryWrapper<EnergyEnvDevice> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.eq("device_status", 1).eq("farm_id", farmId);
            Integer count1 = energyEnvDeviceMapper.selectCount(queryWrapper1);
            Integer OffDeviceCount = count - count1;
            //创建一个数值格式化对象
            NumberFormat numberFormat = NumberFormat.getInstance();
            //设置精确到小数点后两位
            numberFormat.setMaximumFractionDigits(2);
            String onDeviceRate = numberFormat.format((float) count1 / (float) count * 100) + "%";
            jsonObject.put("DeviceCount", count);
            jsonObject.put("OnDeviceCount", count1);
            jsonObject.put("OffDeviceCount", OffDeviceCount);
            jsonObject.put("OnDeviceRate", onDeviceRate);
        }
        return new Result(ResultCode.SUCCESS, jsonObject);
    }

    @Override
    public Result importData(HttpServletRequest httpServletRequest, Map<String, String> paramsMap, MultipartFile files) throws Exception {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String farmName = paramsMap.get("farmName");
        int anInt = Integer.parseInt(unitId);
        Map<String, String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("序号", "id");
        map.put("网关类型", "type");
        map.put("设备编号", "deviceCode");
        map.put("互感器参数", "trans");
        map.put("位置", "deviceName");
        try (
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in, files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<EnergyEnvImport> list = ExcelImportSheet.getListFromExcel(sheet, EnergyEnvImport.class, map);
            for (EnergyEnvImport energyEnvImport : list) {
                if (StringUtils.isNotBlank(energyEnvImport.getDeviceName())) {
                    EnergyEnvDevice envDevice = new EnergyEnvDevice();
                    envDevice.setUnitName(energyEnvImport.getDeviceName());
                    envDevice.setUnitId(anInt);
                    envDevice.setDeviceName(energyEnvImport.getDeviceName());
                    envDevice.setFarmId(Integer.parseInt(farmId));
                    envDevice.setRemark(farmName);
                    if ("电表".equals(energyEnvImport.getType())) {
                        envDevice.setDeviceType(3);
                        envDevice.setTransformer(energyEnvImport.getTrans());
                    } else if ("水表（大口径）".equals(energyEnvImport.getType())) {
                        envDevice.setDeviceType(2);
                    } else if ("水表（小口径）".equals(energyEnvImport.getType())) {
                        envDevice.setDeviceType(1);
                    }
                    String deviceCode = energyEnvImport.getDeviceCode();
                    envDevice.setDeviceCode(deviceCode.substring(deviceCode.length() - 8));
                    energyEnvDeviceMapper.insert(envDevice);
                    anInt++;
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        } finally {
            //写着好看的
        }
        return new Result(10000, "保存成功！", true);
    }

    @Override
    public Result updateName(HttpServletRequest httpServletRequest, MultipartFile files) throws Exception {
        Map<String, String> map = new HashMap<>();
        //表头与键值对的映射关系
        map.put("序号", "id");
        map.put("确认后区域名称", "name");
        map.put("现场核对表号", "deviceCode");
        try (
                //这里面的对象会自动关闭
                InputStream in = files.getInputStream();
                Workbook workbook = ExcelImportSheet.getTypeFromExtends(in, files.getOriginalFilename())
        ) {

            //根据名称获取单张表对象 也可以使用getSheetAt(int index)获取单张表的对象 获取第一张表
            Sheet sheet = workbook.getSheetAt(0);
            List<EnergyUpdateVo> list = ExcelImportSheet.getListFromExcel(sheet, EnergyUpdateVo.class, map);
            QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
            queryWrapper.in("device_type", 1, 2);
            List<EnergyEnvDevice> envDevices = energyEnvDeviceMapper.selectList(queryWrapper);
            for (EnergyUpdateVo energyUpdateVo : list) {
                if (StringUtils.isNotBlank(energyUpdateVo.getDeviceCode())) {
                    for (EnergyEnvDevice envDevice : envDevices) {
                        if (envDevice.getDeviceCode().equals(energyUpdateVo.getDeviceCode())) {
                            envDevice.setDeviceName(energyUpdateVo.getName());
                            envDevice.setUnitName(energyUpdateVo.getName());
                            energyEnvDeviceMapper.updateById(envDevice);
                        }
                    }
                }
            }


        } catch (Exception e) {
            e.printStackTrace();
            return new Result(10001, "数据格式异常！", false);
        } finally {
            //写着好看的
        }
        return new Result(10000, "保存成功！", true);
    }

    @Override
    public void getData() throws Exception {
        List<GetWaterAndElectricityVo> voList = new ArrayList<>();
        List<WaterAndElectricityCount> data = electricityMapper.getData();//水表
        List<WaterAndElectricityCount> data2 = electricityMapper.getData2();//2水表
        for (WaterAndElectricityCount datum : data) {
            for (WaterAndElectricityCount waterAndElectricityCount : data2) {
                if (datum.getDeviceCode().equals(waterAndElectricityCount.getDeviceCode())) {
                    GetWaterAndElectricityVo vo = new GetWaterAndElectricityVo();
                    vo.setFarmName(datum.getFarmName());
                    vo.setTypeName("电表");
                    vo.setDeviceCode(datum.getDeviceCode());
                    vo.setCount1(datum.getValue());
                    vo.setCount2(waterAndElectricityCount.getValue());
                    voList.add(vo);
                }
            }
        }
        Print.printDuck(voList);
    }

    @Override
    public void getDeviceList(String farmId) throws Exception {
        List<EnergyEnvDevice> devices = energyEnvDeviceMapper.selectList(new QueryWrapper<EnergyEnvDevice>().eq("farm_id", farmId));
        List<DeviceListVo> voList = new ArrayList<>();
        for (EnergyEnvDevice device : devices) {
            DeviceListVo deviceListVo = new DeviceListVo();
            if (device.getDeviceType() == 1 || device.getDeviceType() == 2) {
                deviceListVo.setType("水表");
                deviceListVo.setBuildName(device.getDeviceName());
                deviceListVo.setCount(0);
                deviceListVo.setDeviceCode(device.getDeviceCode());
            }
            if (device.getDeviceType()==3) {
                deviceListVo.setType("电表");
                deviceListVo.setBuildName(device.getDeviceName());
                deviceListVo.setCount(device.getTransformer());
                deviceListVo.setDeviceCode(device.getDeviceCode());
            }
            voList.add(deviceListVo);
        }
        Print.printDeviceList(voList);
    }

    @Transactional
    @Override
    public Result updateStart(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId);
        List<EnergyEnvDevice> devices = energyEnvDeviceMapper.selectList(queryWrapper);
        for (EnergyEnvDevice device : devices) {
            if (device.getDeviceType() == 3) {
                //dian
                QueryWrapper<EnergyElectricity> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("chip_code", device.getDeviceCode()).orderByAsc("id").last(" limit 1");
                EnergyElectricity electricity = electricityMapper.selectOne(queryWrapper1);

                QueryWrapper<EnergyElectricity> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("chip_code", device.getDeviceCode()).orderByDesc("id").last(" limit 1");
                EnergyElectricity electricity1 = electricityMapper.selectOne(queryWrapper2);
                if (ObjectUtil.isNotEmpty(electricity)) {
                    if (StringUtils.isNotBlank(electricity.getOther3())) {
                        BigDecimal b1 = new BigDecimal(electricity.getOther3());//最初的数据
                        BigDecimal b2 = new BigDecimal(electricity1.getOther3());
                        BigDecimal subtract = b2.subtract(b1);
                        String s = subtract.toString();
                        System.out.println(s);
                        DecimalFormat def = new DecimalFormat("#.0");
                        electricity1.setOther3(def.format(subtract));
                        electricityMapper.updateById(electricity1);
                    }
                }
            } else {
                //shui
                QueryWrapper<EnergyWater> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.eq("chip_code", device.getDeviceCode()).orderByAsc("id").last(" limit 1");
                EnergyWater electricity = waterMapper.selectOne(queryWrapper1);

                QueryWrapper<EnergyWater> queryWrapper2 = new QueryWrapper<>();
                queryWrapper2.eq("chip_code", device.getDeviceCode()).orderByDesc("id").last(" limit 1");
                EnergyWater electricity1 = waterMapper.selectOne(queryWrapper2);
                if (ObjectUtil.isNotEmpty(electricity)) {
                    if (StringUtils.isNotBlank(electricity.getOther3())) {
                        BigDecimal b1 = new BigDecimal(electricity.getOther3());//最初的数据
                        BigDecimal b2 = new BigDecimal(electricity1.getOther3());
                        BigDecimal subtract = b2.subtract(b1);
                        DecimalFormat def = new DecimalFormat("#.0");
                        electricity1.setOther3(def.format(subtract.toString()));
                        waterMapper.updateById(electricity1);
                    }
                }

            }
        }
        return Result.SUCCESS();
    }

}
