package com.huimv.env.device.listener;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.env.common.entity.BaseFarm;
import com.huimv.env.common.entity.EnergyEnvDevice;
import com.huimv.env.common.entity.Gateway;
import com.huimv.env.common.entity.vo.AllData;
import com.huimv.env.common.mapper.EnergyEnvDeviceMapper;
import com.huimv.env.common.service.IBaseFarmService;
import com.huimv.env.common.service.IGatewayService;
import com.huimv.env.common.utils.Const;
import com.huimv.env.common.utils.MathUtil2;
import com.huimv.env.common.utils.SendSMS;
import com.huimv.env.device.service.DataService;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Map;

@Component
@Slf4j
public class DeviceListener {

    @Autowired
    private DataService dataService;
    @Autowired
    private IGatewayService gatewayService;
    @Autowired
    private IBaseFarmService farmService;
    @Autowired
    private EnergyEnvDeviceMapper deviceMapper;


    @RabbitListener(queues = Const.QUEUE_ENV_SHUI_HUIMV_DATA)
    @RabbitHandler
    public void processReceive(Map<String, String> rawMap) throws ParseException {
//        System.out.println(">>>>>>>>>>>>>> 11 接收数据 RawMap>>" + rawMap.toString());
//        try {
        String askText = rawMap.get("askText");
        AllData allData = JSONUtil.toBean(askText, AllData.class);
        String event = allData.getEvent();
        //时间戳
        String timestamp = allData.getTimestamp();
        //机器id 4823290F01000000 01016B13
        String nodeEui = allData.getContent().getCommonMessage().getNodeEui();
        String gatewayEui = allData.getContent().getCommonMessage().getGatewayEui();
        if ("NODE_DATA_UP".equals(event)) {
            //数据
            String payload = allData.getContent().getNodeUplinkPacket().getNode().getPayload();
            dataService.saveData(payload, timestamp, nodeEui);
            QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
            nodeEui = nodeEui.substring(nodeEui.length() - 8);
            queryWrapper.eq("device_code", nodeEui);
            EnergyEnvDevice envDevice = deviceMapper.selectOne(queryWrapper);
            if (ObjectUtil.isNotEmpty(envDevice)) {
                envDevice.setDeviceStatus(1);
                deviceMapper.updateById(envDevice);
            }

        }

        if ("NODE_ONLINE".equals(event)) {
            //上线通知
            String payload = allData.getContent().getNodeUplinkPacket().getNode().getPayload();
            dataService.saveOnInfo(payload, timestamp, nodeEui);
        }
        if ("NODE_OFFLINE".equals(event)) {
            //离线通知
            String payload = allData.getContent().getNodeUplinkPacket().getNode().getPayload();
            dataService.saveOffInfo(payload, timestamp, nodeEui);
            QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
            nodeEui = nodeEui.substring(nodeEui.length() - 8);
            queryWrapper.eq("device_code", nodeEui);
            EnergyEnvDevice envDevice = deviceMapper.selectOne(queryWrapper);
            if (ObjectUtil.isNotEmpty(envDevice)) {
                envDevice.setDeviceStatus(0);
                deviceMapper.updateById(envDevice);
            }
        }

        if ("GATEWAY_ONLINE".equals(event)) {
            //网关在线线通知
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("gateway", gatewayEui);
            Gateway gateway = gatewayService.getOne(wrapper);
            ArrayList<String> phoneList = new ArrayList<>();
            phoneList.add("13777463003");
            if (gateway.getFarmId() == 24) {
                phoneList.add("15174571875");
            }
            if (ObjectUtil.isNotEmpty(gateway)) {
                String content = gateway.getFarmName() + " 网关:" + gatewayEui + "已在线！";
                SendSMS.sendSMS(phoneList, content);
            } else {
                String content = "未知网关:" + gatewayEui + "已在线！";
                SendSMS.sendSMS(phoneList, content);
            }
        }
        if ("GATEWAY_OFFLINE".equals(event)) {
            //网关在线线通知
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("gateway", gatewayEui);
            Gateway gateway = gatewayService.getOne(wrapper);
            ArrayList<String> phoneList = new ArrayList<>();
            phoneList.add("13777463003");
            if (gateway.getFarmId() == 24) {
                phoneList.add("15174571875");
            }
            if (ObjectUtil.isNotEmpty(gateway)) {
                String content = gateway.getFarmName() + " 网关:" + gatewayEui + "已离线！";
                SendSMS.sendSMS(phoneList, content);
            } else {
                String content = "未知网关:" + gatewayEui + "已离线！";
                SendSMS.sendSMS(phoneList, content);
            }
        }


        System.out.println("数据开始--->" + allData.toString());
//        }catch (Exception e){
//            System.out.println("数据异常"+e);
//        }
    }

}
