package com.huimv.env.device.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.Header;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.auth0.jwt.interfaces.Payload;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.env.common.entity.EnergyElectricity;
import com.huimv.env.common.entity.EnergyEnvDevice;
import com.huimv.env.common.entity.EnergyWater;
import com.huimv.env.common.entity.EnvWarningInfo;
import com.huimv.env.common.mapper.EnvWarningInfoMapper;
import com.huimv.env.common.service.IEnergyElectricityService;
import com.huimv.env.common.service.IEnergyEnvDeviceService;
import com.huimv.env.common.service.IEnergyWaterService;
import com.huimv.env.common.utils.SendSMS;
import com.huimv.env.device.service.DataService;
import org.apache.commons.lang3.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class DataServiceImpl implements DataService {

    @Autowired
    private IEnergyElectricityService energyElectricityService;

    @Autowired
    private IEnergyWaterService energyWaterService;

    @Autowired
    private IEnergyEnvDeviceService energyEnvDeviceService;

    @Autowired
    private EnvWarningInfoMapper warningInfoMapper;

    @Override
    public void saveData(String payload, String timestamp, String nodeEui) throws ParseException {
        // 4823290F01000000 01016B13
        nodeEui = nodeEui.substring(nodeEui.length() - 8);
        //6C0008 01 47 00000025
        //01 水表 ，02电表
        String devType = payload.substring(6, 8);
        //电压 除十
        String voltage = String.format("%.1f", (double) Integer.parseInt(payload.substring(8, 10), 16) / 10.0);
        // 数值
        Integer originalData = Integer.parseInt(payload.substring(10), 16);
        System.out.println(originalData + " = originalData");
        EnergyEnvDevice deviceCode = energyEnvDeviceService.getOne(new QueryWrapper<EnergyEnvDevice>().eq("device_code", nodeEui));
        if (ObjectUtil.isNotEmpty(deviceCode)) {
            Integer farmId = deviceCode.getFarmId();
            Integer unitId = deviceCode.getUnitId();
            //小口径水表
            if ("01".equals(devType) || "02".equals(devType)) {
                saveShui(devType, voltage, originalData, timestamp, nodeEui, payload, farmId, unitId, deviceCode.getDeviceName(), deviceCode.getRemark(), deviceCode.getTransformer(), deviceCode.getDifference());
                QueryWrapper<EnergyWater> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", farmId).eq("chip_code", deviceCode.getDeviceCode()).orderByDesc("id").last(" limit 1");
                EnergyWater energyWater = energyWaterService.getOne(queryWrapper);
                JSONObject paramMap = new JSONObject();
                paramMap.put("name", energyWater.getOther2());
                paramMap.put("address", energyWater.getOther1());
                paramMap.put("number", energyWater.getChipCode());
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                paramMap.put("date", sdf.format(energyWater.getCreateDate()));
                Double aDouble = Double.valueOf(energyWater.getWaterValue());
                if (aDouble > 100) {
                    paramMap.put("dosage", "0");
                } else {
                    paramMap.put("dosage", energyWater.getWaterValue());
                }

                paramMap.put("reading", energyWater.getOther3());
                paramMap.put("type", "水表");
                String result = HttpRequest.post("http://wxdj.htpig.cn:9006/index.php/index/hydropower/hydropowerAdd").header(Header.CONTENT_TYPE, "application/json")
                        .body(paramMap.toJSONString())
                        .timeout(30 * 1000).execute().body();
            }

            if ("03".equals(devType)) {
                saveDian(devType, voltage, originalData, timestamp, nodeEui, payload, farmId, unitId, deviceCode.getDeviceName(), deviceCode.getRemark(), deviceCode.getTransformer(), deviceCode.getDifference());
                QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("farm_id", farmId).eq("chip_code", deviceCode.getDeviceCode()).orderByDesc("id").last(" limit 1");
                EnergyElectricity energyElectricity = energyElectricityService.getOne(queryWrapper);
                JSONObject paramMap = new JSONObject();
                paramMap.put("name", energyElectricity.getOther2());
                paramMap.put("address", energyElectricity.getOther1());
                paramMap.put("number", energyElectricity.getChipCode());
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                paramMap.put("date", sdf.format(energyElectricity.getCreateDate()));
                Double aDouble = Double.valueOf(energyElectricity.getElectricityValue());
                if (aDouble > 1000) {
                    paramMap.put("dosage", "0");
                } else {
                    paramMap.put("dosage", energyElectricity.getElectricityValue());
                }

                paramMap.put("reading", energyElectricity.getOther3());
                paramMap.put("type", "电表");
                String result = HttpRequest.post("http://wxdj.htpig.cn:9006/index.php/index/hydropower/hydropowerAdd").header(Header.CONTENT_TYPE, "application/json")
                        .body(paramMap.toJSONString())
                        .timeout(30 * 1000).execute().body();
            }
        }

    }

    @Override
    public void saveOffInfo(String payload, String timestamp, String nodeEui) {
        //设备编码
        nodeEui = nodeEui.substring(nodeEui.length() - 8);
        //01 水表 ，02电表
        String devType = payload.substring(6, 8);
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("device_code", nodeEui);
        EnergyEnvDevice envDevice = energyEnvDeviceService.getOne(queryWrapper);
        EnvWarningInfo warningInfo = new EnvWarningInfo();
        if (ObjectUtil.isNotEmpty(envDevice)) {
            ArrayList<String> phoneList = new ArrayList<>();
            String content = "";
            if ("01".equals(devType) || "02".equals(devType)) {
                warningInfo.setWarningType(1);//shui
                warningInfo.setWarningContent("水表" + nodeEui + "离线");
                content = "水表" + nodeEui + "离线";
            } else {
                warningInfo.setWarningType(2);
                warningInfo.setWarningContent("电表" + nodeEui + "离线");
                content = "电表" + nodeEui + "离线";
            }
            warningInfo.setBuildLocation(envDevice.getDeviceName());
            warningInfo.setFarmId(envDevice.getFarmId());
            warningInfo.setUserNames(envDevice.getRemark());
            warningInfo.setDate(new Date(Long.parseLong(timestamp)));
            warningInfoMapper.insert(warningInfo);

            phoneList.add("15174571875");
            SendSMS.sendSMS(phoneList, content);

        }

    }

    @Override
    public void saveOnInfo(String payload, String timestamp, String nodeEui) {
        //设备编码
        nodeEui = nodeEui.substring(nodeEui.length() - 8);
        //01 水表 ，02电表
        String devType = payload.substring(6, 8);
        QueryWrapper<EnergyEnvDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("device_code", nodeEui);
        EnergyEnvDevice envDevice = energyEnvDeviceService.getOne(queryWrapper);
        EnvWarningInfo warningInfo = new EnvWarningInfo();
        if (ObjectUtil.isNotEmpty(envDevice)) {
            ArrayList<String> phoneList = new ArrayList<>();
            String content = "";
            if ("01".equals(devType) || "02".equals(devType)) {
                warningInfo.setWarningType(1);//shui
                warningInfo.setWarningContent("水表" + nodeEui + "在线");
                content = "水表" + nodeEui + "在线";
            } else {
                warningInfo.setWarningType(2);
                warningInfo.setWarningContent("电表" + nodeEui + "在线");
                content = "电表" + nodeEui + "在线";
            }
            warningInfo.setBuildLocation(envDevice.getDeviceName());
            warningInfo.setFarmId(envDevice.getFarmId());
            warningInfo.setUserNames(envDevice.getRemark());
            warningInfo.setDate(new Date(Long.parseLong(timestamp)));
            warningInfoMapper.insert(warningInfo);

            phoneList.add("15174571875");
            SendSMS.sendSMS(phoneList, content);
        }

    }

    private void saveDian(String devType, String voltage, Integer originalData, String timestamp, String nodeEui, String payload
            , Integer farmId, Integer unitId, String unitName, String farmName, int transformer, int difference) throws ParseException {
        List<EnergyElectricity> one = energyElectricityService.list(new QueryWrapper<EnergyElectricity>().eq("chip_code", nodeEui).orderByDesc("id").last("limit 1"));
        String lastData = "0";
        Integer str = 0;
        if (one.size() > 0) {
            lastData = one.get(0).getOriginalValue();
        }
        if (originalData == null) {
            originalData = 0;
        }
        if (one.size() == 0) {
            str = 0;
        } else {
            str = (originalData - Integer.parseInt(lastData));
        }
        //差值


        // 将时间戳字符串解析为Date对象
        EnergyElectricity envWater = new EnergyElectricity();
        Date date = new Date();
        envWater.setCreateDate(date);
        envWater.setUploadDate(new Date(Long.parseLong(timestamp)));
        envWater.setFarmId(farmId);
        envWater.setUnitId(unitId);
        envWater.setOriginalValue((originalData - difference) + "");
        envWater.setChipCode(nodeEui);
        envWater.setOriginalDate(payload);
        envWater.setOther1(unitName);
        envWater.setOther2(farmName);

        envWater.setDeviceType(Integer.parseInt(devType));

        BigDecimal bigDecimal = BigDecimal.valueOf(0);
        if (one.size() > 0) {
            bigDecimal = new BigDecimal(one.get(0).getOther3());
        }

        if (str < 0) {
            envWater.setElectricityValue(String.format("%.1f", (double) originalData * transformer / 10.0));
            BigDecimal bigDecimal1 = new BigDecimal(String.format("%.1f", (double) originalData * transformer / 10.0));
            envWater.setOther3(bigDecimal.add(bigDecimal1).toString());
        } else if (str == 0) {
            envWater.setElectricityValue("0");
            envWater.setOther3(bigDecimal.toString());
        } else {
            envWater.setElectricityValue(String.format("%.1f", (double) str * transformer / 10.0));
            BigDecimal bigDecimal1 = new BigDecimal(String.format("%.1f", (double) str * transformer / 10.0));
            envWater.setOther3(bigDecimal.add(bigDecimal1).toString());
        }
        envWater.setVoltage(voltage);
        energyElectricityService.save(envWater);
    }

    private void saveShui(String devType, String voltage, Integer originalData, String timestamp, String nodeEui
            , String payload, Integer farmId, Integer unitId, String unitName, String farmName, int transformer, int difference) throws ParseException {
        List<EnergyWater> one = energyWaterService.list(new QueryWrapper<EnergyWater>().eq("chip_code", nodeEui).orderByDesc("id").last("limit 1"));
        String lastData = "0";
        Integer str = 0;
        if (ObjectUtil.isNotEmpty(one)) {
            lastData = one.get(0).getOriginalValue();
            str = (originalData - Integer.parseInt(lastData) - difference);
            if (str < 0) {
                str = (originalData - Integer.parseInt(lastData));
            }
        } else {
            str = 0;
        }


        //差值


        // 将时间戳字符串解析为Date对象
        EnergyWater envWater = new EnergyWater();
        Date date = new Date();
        envWater.setCreateDate(date);
        envWater.setDeviceType(Integer.parseInt(devType));
        envWater.setUploadDate(new Date(Long.parseLong(timestamp)));
        envWater.setFarmId(farmId);
        envWater.setOriginalValue((originalData - difference) + "");
        envWater.setChipCode(nodeEui);
        envWater.setUnitId(unitId);
        envWater.setOther1(unitName);
        envWater.setOther2(farmName);
        envWater.setOriginalDate(payload);
        BigDecimal bigDecimal = BigDecimal.valueOf(0);
        if (one.size() > 0) {
            if (!"".equals(one.get(0).getOther3()) && null != one.get(0).getOther3()) {
                bigDecimal = new BigDecimal(one.get(0).getOther3());
            }

        }
        if (str < 0) {
            envWater.setWaterValue(String.format("%.1f", (double) originalData * transformer / 10.0));
            BigDecimal bigDecimal1 = new BigDecimal(String.format("%.1f", (double) originalData * transformer / 10.0));
            envWater.setOther3(bigDecimal.add(bigDecimal1).toString());
        } else if (str == 0) {
            envWater.setWaterValue("0");
            envWater.setOther3(bigDecimal.toString());
        } else {
            envWater.setWaterValue(String.format("%.1f", (double) str * transformer / 10.0));
            BigDecimal bigDecimal1 = new BigDecimal(String.format("%.1f", (double) str * transformer / 10.0));
            envWater.setOther3(bigDecimal.add(bigDecimal1).toString());
        }

        envWater.setVoltage(voltage);

        energyWaterService.save(envWater);

    }

    public static void main(String[] args) {

//       String  timestamp =  ;
//        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd HHmmss");
        System.out.println(new Date(Long.parseLong("1694678879843")));

    }


}
